@@ -10,7 +10,7 @@ called from the command line.
     import markdown
     html = markdown.markdown(your_text_string)
 
-See <http://www.freewisdom.org/projects/python-markdown/> for more
+See <http://packages.python.org/Markdown/> for more
 information and instructions on how to extend the functionality of
 Python Markdown.  Read that before you try modifying this file.
 
@@ -22,7 +22,7 @@ Limberg](http://achinghead.com/) and [Artem Yunusov](http://blog.splyer.com).
 
 Contact: markdown@freewisdom.org
 
-Copyright 2007-2012 The Python Markdown Project (v. 1.7 and later)
+Copyright 2007-2013 The Python Markdown Project (v. 1.7 and later)
 Copyright 200? Django Software Foundation (OrderedDict implementation)
 Copyright 2004, 2005, 2006 Yuri Takhteyev (v. 0.2-1.6b)
 Copyright 2004 Manfred Stienstra (the original version)
@@ -30,32 +30,31 @@ Copyright 2004 Manfred Stienstra (the original version)
 License: BSD (see LICENSE for details).
 """
 
-version = "2.1.1"
-version_info = (2,1,1, "final")
-
-import re
+from __future__ import absolute_import
+from __future__ import unicode_literals
+from .__version__ import version, version_info
 import codecs
 import sys
 import logging
-import util
-from preprocessors import build_preprocessors
-from blockprocessors import build_block_parser
-from treeprocessors import build_treeprocessors
-from inlinepatterns import build_inlinepatterns
-from postprocessors import build_postprocessors
-from extensions import Extension
-from serializers import to_html_string, to_xhtml_string
+from . import util
+from .preprocessors import build_preprocessors
+from .blockprocessors import build_block_parser
+from .treeprocessors import build_treeprocessors
+from .inlinepatterns import build_inlinepatterns
+from .postprocessors import build_postprocessors
+from .extensions import Extension
+from .serializers import to_html_string, to_xhtml_string
 
 __all__ = ['Markdown', 'markdown', 'markdownFromFile']
 
 logger = logging.getLogger('MARKDOWN')
 
 
-class Markdown:
+class Markdown(object):
     """Convert Markdown to HTML."""
 
     doc_tag = "div"     # Element used to wrap document - later removed
-    
+
     option_defaults = {
         'html_replacement_text' : '[HTML_REMOVED]',
         'tab_length'            : 4,
@@ -63,7 +62,7 @@ class Markdown:
         'smart_emphasis'        : True,
         'lazy_ol'               : True,
     }
-    
+
     output_formats = {
         'html'  : to_html_string,
         'html4' : to_html_string,
@@ -73,9 +72,9 @@ class Markdown:
         'xhtml5': to_xhtml_string,
     }
 
-    ESCAPED_CHARS = ['\\', '`', '*', '_', '{', '}', '[', ']', 
+    ESCAPED_CHARS = ['\\', '`', '*', '_', '{', '}', '[', ']',
                     '(', ')', '>', '#', '+', '-', '.', '!']
-    
+
     def __init__(self, *args, **kwargs):
         """
         Creates a new Markdown instance.
@@ -86,7 +85,7 @@ class Markdown:
            If they are of type string, the module mdx_name.py will be loaded.
            If they are a subclass of markdown.Extension, they will be used
            as-is.
-        * extension-configs: Configuration settingis for extensions.
+        * extension_configs: Configuration settingis for extensions.
         * output_format: Format of output. Supported formats are:
             * "xhtml1": Outputs XHTML 1.x. Default.
             * "xhtml5": Outputs XHTML style tags of HTML 5
@@ -101,16 +100,16 @@ class Markdown:
         * html_replacement_text: Text used when safe_mode is set to "replace".
         * tab_length: Length of tabs in the source. Default: 4
         * enable_attributes: Enable the conversion of attributes. Default: True
-        * smart_emphasis: Treat `_connected_words_` intelegently Default: True
+        * smart_emphasis: Treat `_connected_words_` intelligently Default: True
         * lazy_ol: Ignore number of first item of ordered lists. Default: True
 
         """
 
-        # For backward compatability, loop through old positional args
+        # For backward compatibility, loop through old positional args
         pos = ['extensions', 'extension_configs', 'safe_mode', 'output_format']
         c = 0
         for arg in args:
-            if not kwargs.has_key(pos[c]):
+            if pos[c] not in kwargs:
                 kwargs[pos[c]] = arg
             c += 1
             if c == len(pos):
@@ -119,9 +118,13 @@ class Markdown:
 
         # Loop through kwargs and assign defaults
         for option, default in self.option_defaults.items():
-            setattr(self, option, kwargs.get(option, default)) 
+            setattr(self, option, kwargs.get(option, default))
 
         self.safeMode = kwargs.get('safe_mode', False)
+        if self.safeMode and 'enable_attributes' not in kwargs:
+            # Disable attributes in safeMode when not explicitly set
+            self.enable_attributes = False
+
         self.registeredExtensions = []
         self.docType = ""
         self.stripTopLevelTags = True
@@ -138,7 +141,7 @@ class Markdown:
     def build_parser(self):
         """ Build the parser from the various parts. """
         self.preprocessors = build_preprocessors(self)
-        self.parser = build_block_parser(self) 
+        self.parser = build_block_parser(self)
         self.inlinePatterns = build_inlinepatterns(self)
         self.treeprocessors = build_treeprocessors(self)
         self.postprocessors = build_postprocessors(self)
@@ -156,13 +159,13 @@ class Markdown:
 
         """
         for ext in extensions:
-            if isinstance(ext, basestring):
+            if isinstance(ext, util.string_type):
                 ext = self.build_extension(ext, configs.get(ext, []))
             if isinstance(ext, Extension):
-                # might raise NotImplementedError, but that's the extension author's problem
                 ext.extendMarkdown(self, globals())
-            else:
-                raise ValueError('Extension "%s.%s" must be of type: "markdown.Extension".' \
+            elif ext is not None:
+                raise TypeError(
+                    'Extension "%s.%s" must be of type: "markdown.Extension"'
                     % (ext.__class__.__module__, ext.__class__.__name__))
 
         return self
@@ -184,31 +187,35 @@ class Markdown:
             pairs = [x.split("=") for x in ext_args.split(",")]
             configs.update([(x.strip(), y.strip()) for (x, y) in pairs])
 
-        # Setup the module names
-        ext_module = 'markdown.extensions'
-        module_name_new_style = '.'.join([ext_module, ext_name])
-        module_name_old_style = '_'.join(['mdx', ext_name])
+        # Setup the module name
+        module_name = ext_name
+        if '.' not in ext_name:
+            module_name = '.'.join(['markdown.extensions', ext_name])
 
         # Try loading the extension first from one place, then another
-        try: # New style (markdown.extensons.<extension>)
-            module = __import__(module_name_new_style, {}, {}, [ext_module])
+        try: # New style (markdown.extensions.<extension>)
+            module = __import__(module_name, {}, {}, [module_name.rpartition('.')[0]])
         except ImportError:
+            module_name_old_style = '_'.join(['mdx', ext_name])
             try: # Old style (mdx_<extension>)
                 module = __import__(module_name_old_style)
-            except ImportError:
-                logger.warn("Failed loading extension '%s' from '%s' or '%s'"
-                    % (ext_name, module_name_new_style, module_name_old_style))
-                # Return None so we don't try to initiate none-existant extension
-                return None
+            except ImportError as e:
+                message = "Failed loading extension '%s' from '%s' or '%s'" \
+                    % (ext_name, module_name, module_name_old_style)
+                e.args = (message,) + e.args[1:]
+                raise
 
         # If the module is loaded successfully, we expect it to define a
         # function called makeExtension()
         try:
             return module.makeExtension(configs.items())
-        except AttributeError, e:
-            logger.warn("Failed to initiate extension '%s': %s" % (ext_name, e))
-            return None
-    
+        except AttributeError as e:
+            message = e.args[0]
+            message = "Failed to initiate extension " \
+                      "'%s': %s" % (ext_name, message)
+            e.args = (message,) + e.args[1:]
+            raise
+
     def registerExtension(self, extension):
         """ This gets called by the extension """
         self.registeredExtensions.append(extension)
@@ -229,11 +236,17 @@ class Markdown:
 
     def set_output_format(self, format):
         """ Set the output format for the class instance. """
+        self.output_format = format.lower()
         try:
-            self.serializer = self.output_formats[format.lower()]
-        except KeyError:
-            raise KeyError('Invalid Output Format: "%s". Use one of %s.' \
-                               % (format, self.output_formats.keys()))
+            self.serializer = self.output_formats[self.output_format]
+        except KeyError as e:
+            valid_formats = list(self.output_formats.keys())
+            valid_formats.sort()
+            message = 'Invalid Output Format: "%s". Use one of %s.' \
+                       % (self.output_format, 
+                          '"' + '", "'.join(valid_formats) + '"')
+            e.args = (message,) + e.args[1:]
+            raise
         return self
 
     def convert(self, source):
@@ -249,10 +262,10 @@ class Markdown:
         1. A bunch of "preprocessors" munge the input text.
         2. BlockParser() parses the high-level structural elements of the
            pre-processed text into an ElementTree.
-        3. A bunch of "treeprocessors" are run against the ElementTree. One 
-           such treeprocessor runs InlinePatterns against the ElementTree, 
+        3. A bunch of "treeprocessors" are run against the ElementTree. One
+           such treeprocessor runs InlinePatterns against the ElementTree,
            detecting inline markup.
-        4. Some post-processors are run against the text after the ElementTree 
+        4. Some post-processors are run against the text after the ElementTree
            has been serialized into text.
         5. The output is written to a string.
 
@@ -260,20 +273,15 @@ class Markdown:
 
         # Fixup the source text
         if not source.strip():
-            return u""  # a blank unicode string
-        
+            return ''  # a blank unicode string
+
         try:
-            source = unicode(source)
-        except UnicodeDecodeError, e:
+            source = util.text_type(source)
+        except UnicodeDecodeError as e:
             # Customise error message while maintaining original trackback
             e.reason += '. -- Note: Markdown only accepts unicode input!'
             raise
 
-        source = source.replace(util.STX, "").replace(util.ETX, "")
-        source = source.replace("\r\n", "\n").replace("\r", "\n") + "\n\n"
-        source = re.sub(r'\n\s+\n', '\n\n', source)
-        source = source.expandtabs(self.tab_length)
-
         # Split into lines and run the line preprocessors.
         self.lines = source.split("\n")
         for prep in self.preprocessors.values():
@@ -285,7 +293,7 @@ class Markdown:
         # Run the tree-processors
         for treeprocessor in self.treeprocessors.values():
             newRoot = treeprocessor.run(root)
-            if newRoot:
+            if newRoot is not None:
                 root = newRoot
 
         # Serialize _properly_.  Strip top-level tags.
@@ -334,7 +342,7 @@ class Markdown:
 
         # Read the source
         if input:
-            if isinstance(input, str):
+            if isinstance(input, util.string_type):
                 input_file = codecs.open(input, mode="r", encoding=encoding)
             else:
                 input_file = codecs.getreader(encoding)(input)
@@ -342,7 +350,7 @@ class Markdown:
             input_file.close()
         else:
             text = sys.stdin.read()
-            if not isinstance(text, unicode):
+            if not isinstance(text, util.text_type):
                 text = text.decode(encoding)
 
         text = text.lstrip('\ufeff') # remove the byte-order mark
@@ -352,9 +360,9 @@ class Markdown:
 
         # Write to file or stdout
         if output:
-            if isinstance(output, str):
-                output_file = codecs.open(output, "w", 
-                                          encoding=encoding, 
+            if isinstance(output, util.string_type):
+                output_file = codecs.open(output, "w",
+                                          encoding=encoding,
                                           errors="xmlcharrefreplace")
                 output_file.write(html)
                 output_file.close()
@@ -364,7 +372,14 @@ class Markdown:
                 output_file.write(html)
                 # Don't close here. User may want to write more.
         else:
-            sys.stdout.write(html)
+            # Encode manually and write bytes to stdout. 
+            html = html.encode(encoding, "xmlcharrefreplace")
+            try:
+                # Write bytes directly to buffer (Python 3).
+                sys.stdout.buffer.write(html)
+            except AttributeError:
+                # Probably Python 2, which works with bytes by default.
+                sys.stdout.write(html)
 
         return self
 
@@ -398,30 +413,30 @@ def markdown(text, *args, **kwargs):
 
 def markdownFromFile(*args, **kwargs):
     """Read markdown code from a file and write it to a file or a stream.
-    
+
     This is a shortcut function which initializes an instance of Markdown,
     and calls the convertFile method rather than convert.
-    
+
     Keyword arguments:
-    
+
     * input: a file name or readable object.
     * output: a file name or writable object.
     * encoding: Encoding of input and output.
     * Any arguments accepted by the Markdown class.
-    
+
     """
     # For backward compatibility loop through positional args
     pos = ['input', 'output', 'extensions', 'encoding']
     c = 0
     for arg in args:
-        if not kwargs.has_key(pos[c]):
+        if pos[c] not in kwargs:
             kwargs[pos[c]] = arg
         c += 1
         if c == len(pos):
             break
 
     md = Markdown(**kwargs)
-    md.convertFile(kwargs.get('input', None), 
+    md.convertFile(kwargs.get('input', None),
                    kwargs.get('output', None),
                    kwargs.get('encoding', None))
 
