@@ -1,5 +1,3 @@
-#!/usr/bin/env python
-
 """
 Fenced Code Extension for Python Markdown
 =========================================
@@ -61,6 +59,20 @@ Optionally backticks instead of tildes as per how github's code block markdown i
     ~~~~~ # these tildes will not close the block
     </code></pre>
 
+If the codehighlite extension and Pygments are installed, lines can be highlighted:
+
+    >>> text = '''
+    ... ```hl_lines="1 3"
+    ... line 1
+    ... line 2
+    ... line 3
+    ... ```'''
+    >>> print markdown.markdown(text, extensions=['codehilite', 'fenced_code'])
+    <pre><code><span class="hilight">line 1</span>
+    line 2
+    <span class="hilight">line 3</span>
+    </code></pre>
+
 Copyright 2007-2008 [Waylan Limberg](http://achinghead.com/).
 
 Project website: <http://packages.python.org/Markdown/extensions/fenced_code_blocks.html>
@@ -75,19 +87,15 @@ Dependencies:
 
 """
 
+from __future__ import absolute_import
+from __future__ import unicode_literals
+from . import Extension
+from ..preprocessors import Preprocessor
+from .codehilite import CodeHilite, CodeHiliteExtension, parse_hl_lines
 import re
-import markdown
-from markdown.extensions.codehilite import CodeHilite, CodeHiliteExtension
 
-# Global vars
-FENCED_BLOCK_RE = re.compile( \
-    r'(?P<fence>^(?:~{3,}|`{3,}))[ ]*(\{?\.?(?P<lang>[a-zA-Z0-9_+-]*)\}?)?[ ]*\n(?P<code>.*?)(?<=\n)(?P=fence)[ ]*$',
-    re.MULTILINE|re.DOTALL
-    )
-CODE_WRAP = '<pre><code%s>%s</code></pre>'
-LANG_TAG = ' class="%s"'
 
-class FencedCodeExtension(markdown.Extension):
+class FencedCodeExtension(Extension):
 
     def extendMarkdown(self, md, md_globals):
         """ Add FencedBlockPreprocessor to the Markdown instance. """
@@ -95,13 +103,23 @@ class FencedCodeExtension(markdown.Extension):
 
         md.preprocessors.add('fenced_code_block',
                                  FencedBlockPreprocessor(md),
-                                 "_begin")
+                                 ">normalize_whitespace")
 
 
-class FencedBlockPreprocessor(markdown.preprocessors.Preprocessor):
+class FencedBlockPreprocessor(Preprocessor):
+    FENCED_BLOCK_RE = re.compile(r'''
+(?P<fence>^(?:~{3,}|`{3,}))[ ]*         # Opening ``` or ~~~
+(\{?\.?(?P<lang>[a-zA-Z0-9_+-]*))?[ ]*  # Optional {, and lang
+# Optional highlight lines, single- or double-quote-delimited
+(hl_lines=(?P<quot>"|')(?P<hl_lines>.*?)(?P=quot))?[ ]*
+}?[ ]*\n                                # Optional closing }
+(?P<code>.*?)(?<=\n)
+(?P=fence)[ ]*$''', re.MULTILINE | re.DOTALL | re.VERBOSE)
+    CODE_WRAP = '<pre><code%s>%s</code></pre>'
+    LANG_TAG = ' class="%s"'
 
     def __init__(self, md):
-        markdown.preprocessors.Preprocessor.__init__(self, md)
+        super(FencedBlockPreprocessor, self).__init__(md)
 
         self.checked_for_codehilite = False
         self.codehilite_conf = {}
@@ -120,26 +138,27 @@ class FencedBlockPreprocessor(markdown.preprocessors.Preprocessor):
 
         text = "\n".join(lines)
         while 1:
-            m = FENCED_BLOCK_RE.search(text)
+            m = self.FENCED_BLOCK_RE.search(text)
             if m:
                 lang = ''
                 if m.group('lang'):
-                    lang = LANG_TAG % m.group('lang')
+                    lang = self.LANG_TAG % m.group('lang')
 
                 # If config is not empty, then the codehighlite extension
-                # is enabled, so we call it to highlite the code
+                # is enabled, so we call it to highlight the code
                 if self.codehilite_conf:
                     highliter = CodeHilite(m.group('code'),
-                            linenos=self.codehilite_conf['force_linenos'][0],
+                            linenums=self.codehilite_conf['linenums'][0],
                             guess_lang=self.codehilite_conf['guess_lang'][0],
                             css_class=self.codehilite_conf['css_class'][0],
                             style=self.codehilite_conf['pygments_style'][0],
                             lang=(m.group('lang') or None),
-                            noclasses=self.codehilite_conf['noclasses'][0])
+                            noclasses=self.codehilite_conf['noclasses'][0],
+                            hl_lines=parse_hl_lines(m.group('hl_lines')))
 
                     code = highliter.hilite()
                 else:
-                    code = CODE_WRAP % (lang, self._escape(m.group('code')))
+                    code = self.CODE_WRAP % (lang, self._escape(m.group('code')))
 
                 placeholder = self.markdown.htmlStash.store(code, safe=True)
                 text = '%s\n%s\n%s'% (text[:m.start()], placeholder, text[m.end():])
@@ -158,8 +177,3 @@ class FencedBlockPreprocessor(markdown.preprocessors.Preprocessor):
 
 def makeExtension(configs=None):
     return FencedCodeExtension(configs=configs)
-
-
-if __name__ == "__main__":
-    import doctest
-    doctest.testmod()
